@@ -7,8 +7,10 @@ import asyncio
 import itertools
 import logging
 import socket
+import sys
 import time
 import warnings
+from asyncio.transports import Transport
 from base64 import b64encode
 from contextlib import suppress
 from smtplib import (
@@ -19,7 +21,7 @@ from smtplib import (
     SMTPServerDisconnected,
 )
 from textwrap import dedent
-from typing import Any, AnyStr, Callable, Generator, List, Tuple
+from typing import cast, Any, Callable, Generator, List, Tuple, Union
 
 import pytest
 from pytest_mock import MockFixture
@@ -51,6 +53,7 @@ CRLF = "\r\n"
 BCRLF = b"\r\n"
 MAIL_LOG = logging.getLogger("mail.log")
 MAIL_LOG.setLevel(logging.DEBUG)
+B64EQUALS = b64encode(b"=").decode()
 
 # fh = logging.FileHandler("~smtp.log")
 # fh.setFormatter(logging.Formatter("{asctime} - {levelname} - {message}", style="{"))
@@ -62,10 +65,7 @@ MAIL_LOG.setLevel(logging.DEBUG)
 
 
 def auth_callback(mechanism, login, password) -> bool:
-    if login and login.decode() == "goodlogin":
-        return True
-    else:
-        return False
+    return login and login.decode() == "goodlogin"
 
 
 def assert_nopassleak(passwd: str, record_tuples: List[Tuple[str, int, str]]):
@@ -87,7 +87,7 @@ class UndescribableError(Exception):
 class ErrorSMTP(Server):
     exception_type = ValueError
 
-    async def smtp_HELO(self, hostname):
+    async def smtp_HELO(self, hostname: str):
         raise self.exception_type("test")
 
 
@@ -100,10 +100,10 @@ class ErrorSMTP(Server):
 # noinspection TimingAttack
 class PeekerHandler:
     sess: SMTPSession = None
-    login: AnyStr = None
+    login: Union[str, bytes, None] = None
     login_data: Any = None
-    mechanism: AnyStr = None
-    password: AnyStr = None
+    mechanism: Union[str, bytes, None] = None
+    password: Union[str, bytes, None] = None
 
     # Please do not insert "_" after auth; that will 'fool' SMTP into thinking this is
     # an AUTH Mechanism, and we totally do NOT want that.
@@ -136,8 +136,13 @@ class PeekerHandler:
             return AuthResult(success=True, auth_data=login_data)
 
     async def handle_MAIL(
-        self, server, session: SMTPSession, envelope, address, mail_options
-    ):
+        self,
+        server: Server,
+        session: SMTPSession,
+        envelope: SMTPEnvelope,
+        address: str,
+        mail_options: dict,
+    ) -> str:
         self.sess = session
         return S.S250_OK.to_str()
 
@@ -157,7 +162,7 @@ class PeekerHandler:
     async def auth_DONT(self, server, args):
         return MISSING
 
-    async def auth_WITH_UNDERSCORE(self, server: Server, args):
+    async def auth_WITH_UNDERSCORE(self, server: Server, args) -> str:
         """
         Be careful when using this AUTH mechanism; log_client_response is set to
         True, and this will raise some severe warnings.
@@ -180,7 +185,9 @@ class StoreEnvelopeOnVRFYHandler:
 
     envelope = None
 
-    async def handle_VRFY(self, server, session, envelope, addr):
+    async def handle_VRFY(
+        self, server: Server, session: SMTPSession, envelope: SMTPEnvelope, addr: str
+    ) -> str:
         self.envelope = envelope
         return S.S250_OK.to_str()
 
@@ -189,10 +196,10 @@ class ErroringHandler:
     error = None
     custom_response = False
 
-    async def handle_DATA(self, server, session, envelope):
+    async def handle_DATA(self, server, session, envelope) -> str:
         return "499 Could not accept the message"
 
-    async def handle_exception(self, error):
+    async def handle_exception(self, error) -> str:
         self.error = error
         if not self.custom_response:
             return "500 ErroringHandler handling error"
@@ -215,7 +222,7 @@ class ErroringHandlerConnectionLost:
 class ErroringErrorHandler:
     error = None
 
-    async def handle_exception(self, error):
+    async def handle_exception(self, error: Exception):
         self.error = error
         raise ValueError("ErroringErrorHandler test")
 
@@ -223,13 +230,19 @@ class ErroringErrorHandler:
 class UndescribableErrorHandler:
     error = None
 
-    async def handle_exception(self, error):
+    async def handle_exception(self, error: Exception):
         self.error = error
         raise UndescribableError()
 
 
 class SleepingHeloHandler:
-    async def handle_HELO(self, server, session, envelope, hostname):
+    async def handle_HELO(
+        self,
+        server: Server,
+        session: SMTPSession,
+        envelope: SMTPEnvelope,
+        hostname: str,
+    ) -> str:
         await asyncio.sleep(0.01)
         session.host_name = hostname
         return "250 {}".format(server.hostname)
@@ -267,8 +280,7 @@ class CustomIdentController(Controller):
     ident: bytes = b"Identifying SMTP v2112"
 
     def factory(self):
-        server = Server(self.handler, ident=self.ident.decode())
-        return server
+        return Server(self.handler, ident=self.ident.decode())
 
 
 # endregion
@@ -278,18 +290,19 @@ class CustomIdentController(Controller):
 
 
 @pytest.fixture
-def transport_resp(mocker: MockFixture):
+def transport_resp(mocker: MockFixture) -> Tuple[Transport, list]:
     responses = []
     mocked = mocker.Mock()
     mocked.write = responses.append
     #
-    yield mocked, responses
+    return cast(Transport, mocked), responses
 
 
 @pytest.fixture
 def get_protocol(
-    temp_event_loop, transport_resp
-) -> Generator[Callable[..., Server], None, None]:
+    temp_event_loop: asyncio.AbstractEventLoop,
+    transport_resp: Any,
+) -> Callable[..., Server]:
     transport, _ = transport_resp
 
     def getter(*args, **kwargs) -> Server:
@@ -297,14 +310,16 @@ def get_protocol(
         proto.connection_made(transport)
         return proto
 
-    yield getter
+    return getter
 
 
 # region #### Fixtures: Controllers ##################################################
 
 
 @pytest.fixture
-def auth_peeker_controller(get_controller) -> Generator[Controller, None, None]:
+def auth_peeker_controller(
+    get_controller: Callable[..., Controller]
+) -> Generator[Controller, None, None]:
     handler = PeekerHandler()
     controller = get_controller(
         handler,
@@ -324,7 +339,7 @@ def auth_peeker_controller(get_controller) -> Generator[Controller, None, None]:
 
 @pytest.fixture
 def authenticator_peeker_controller(
-    get_controller,
+    get_controller: Callable[..., Controller]
 ) -> Generator[Controller, None, None]:
     handler = PeekerHandler()
     controller = get_controller(
@@ -345,7 +360,8 @@ def authenticator_peeker_controller(
 
 @pytest.fixture
 def decoding_authnotls_controller(
-    get_handler, get_controller
+    get_handler: Callable,
+    get_controller: Callable[..., Controller]
 ) -> Generator[Controller, None, None]:
     handler = get_handler()
     controller = get_controller(
@@ -368,7 +384,7 @@ def decoding_authnotls_controller(
 
 
 @pytest.fixture
-def error_controller(get_handler) -> Generator[ErrorController, None, None]:
+def error_controller(get_handler: Callable) -> Generator[ErrorController, None, None]:
     handler = get_handler()
     controller = ErrorController(handler)
     controller.start()
@@ -417,10 +433,8 @@ class TestProtocol:
                 ]
             )
         )
-        try:
+        with suppress(asyncio.CancelledError):
             temp_event_loop.run_until_complete(protocol._handler_coroutine)
-        except asyncio.CancelledError:
-            pass
         _, responses = transport_resp
         assert responses[5] == S.S250_OK.to_bytes() + b"\r\n"
         assert len(handler.box) == 1
@@ -441,10 +455,8 @@ class TestProtocol:
                 ]
             )
         )
-        try:
+        with suppress(asyncio.CancelledError):
             temp_event_loop.run_until_complete(protocol._handler_coroutine)
-        except asyncio.CancelledError:
-            pass
         _, responses = transport_resp
         assert responses[5] == S.S250_OK.to_bytes() + b"\r\n"
         assert len(handler.box) == 1
@@ -974,7 +986,7 @@ class TestSMTPAuth(_CommonMethods):
         resp = client.docmd("AUTH WITH_UNDERSCORE")
         assert resp == (334, b"challenge")
         with warnings.catch_warnings(record=True) as w:
-            assert client.docmd("=") == S.S235_AUTH_SUCCESS
+            assert client.docmd(B64EQUALS) == S.S235_AUTH_SUCCESS
         assert len(w) > 0
         assert str(w[0].message) == "AUTH interaction logging is enabled!"
         assert str(w[1].message) == "Sensitive information might be leaked!"
@@ -986,19 +998,19 @@ class TestAuthMechanisms(_CommonMethods):
     @pytest.fixture
     def do_auth_plain1(
         self, client
-    ) -> Generator[Callable[[str], Tuple[int, bytes]], None, None]:
+    ) -> Callable[[str], Tuple[int, bytes]]:
         self._ehlo(client)
 
         def do(param: str) -> Tuple[int, bytes]:
             return client.docmd("AUTH PLAIN " + param)
 
         do.client = client
-        yield do
+        return do
 
     @pytest.fixture
     def do_auth_login3(
         self, client
-    ) -> Generator[Callable[[str], Tuple[int, bytes]], None, None]:
+    ) -> Callable[[str], Tuple[int, bytes]]:
         self._ehlo(client)
         resp = client.docmd("AUTH LOGIN")
         assert resp == S.S334_AUTH_USERNAME
@@ -1007,7 +1019,7 @@ class TestAuthMechanisms(_CommonMethods):
             return client.docmd(param)
 
         do.client = client
-        yield do
+        return do
 
     def test_ehlo(self, client):
         code, mesg = client.ehlo("example.com")
@@ -1050,7 +1062,12 @@ class TestAuthMechanisms(_CommonMethods):
         client.user = "goodlogin"
         client.password = PW
         auth_meth = getattr(client, "auth_" + mechanism)
-        if (mechanism, init_resp) == ("login", False):
+        if (mechanism, init_resp) == ("login", False) and (
+                sys.version_info < (3, 8, 9)
+                or (3, 9, 0) < sys.version_info < (3, 9, 4)):
+            # The bug with SMTP.auth_login was fixed in Python 3.10 and backported
+            # to 3.9.4 and and 3.8.9.
+            # See https://github.com/python/cpython/pull/24118 for the fixes.:
             with pytest.raises(SMTPAuthenticationError):
                 client.auth(mechanism, auth_meth, initial_response_ok=init_resp)
             client.docmd("*")
@@ -1083,7 +1100,7 @@ class TestAuthMechanisms(_CommonMethods):
         assert resp == S.S535_AUTH_INVALID
 
     def test_plain1_empty(self, do_auth_plain1):
-        resp = do_auth_plain1("=")
+        resp = do_auth_plain1(B64EQUALS)
         assert resp == S.S501_AUTH_CANTSPLIT
 
     def test_plain1_good_credentials(
@@ -1119,11 +1136,11 @@ class TestAuthMechanisms(_CommonMethods):
         assert_nopassleak(PW, caplog.record_tuples)
 
     @pytest.fixture
-    def client_auth_plain2(self, client) -> Generator[SMTPClient, None, None]:
+    def client_auth_plain2(self, client) -> SMTPClient:
         self._ehlo(client)
         resp = client.docmd("AUTH PLAIN")
         assert resp == S.S334_AUTH_EMPTYPROMPT
-        yield client
+        return client
 
     def test_plain2_good_credentials(
         self, caplog, auth_peeker_controller, client_auth_plain2
@@ -1145,7 +1162,7 @@ class TestAuthMechanisms(_CommonMethods):
         assert resp == S.S535_AUTH_INVALID
 
     def test_plain2_no_credentials(self, client_auth_plain2):
-        resp = client_auth_plain2.docmd("=")
+        resp = client_auth_plain2.docmd(B64EQUALS)
         assert resp == S.S501_AUTH_CANTSPLIT
 
     def test_plain2_abort(self, client_auth_plain2):
@@ -1214,9 +1231,9 @@ class TestAuthMechanisms(_CommonMethods):
         assert resp == S.S535_AUTH_INVALID
 
     def test_login3_empty_credentials(self, do_auth_login3):
-        resp = do_auth_login3("=")
+        resp = do_auth_login3(B64EQUALS)
         assert resp == S.S334_AUTH_PASSWORD
-        resp = do_auth_login3("=")
+        resp = do_auth_login3(B64EQUALS)
         assert resp == S.S535_AUTH_INVALID
 
     def test_login3_abort_username(self, do_auth_login3):
@@ -1224,7 +1241,7 @@ class TestAuthMechanisms(_CommonMethods):
         assert resp == S.S501_AUTH_ABORTED
 
     def test_login3_abort_password(self, do_auth_login3):
-        resp = do_auth_login3("=")
+        resp = do_auth_login3(B64EQUALS)
         assert resp == S.S334_AUTH_PASSWORD
         resp = do_auth_login3("*")
         assert resp == S.S501_AUTH_ABORTED
@@ -1485,7 +1502,7 @@ class TestSMTPWithController(_CommonMethods):
                     From: anne@example.com
                     To: bart@example.com
                     Subjebgct: A test
-                    
+
                     Testing
                 """
                 ),
@@ -1930,12 +1947,16 @@ class TestAuthArgs:
     def test_warn_authreqnotls(self, caplog):
         with pytest.warns(UserWarning) as record:
             _ = Server(Sink(), auth_required=True, auth_require_tls=False)
-        assert len(record) == 1
-        assert (
-            record[0].message.args[0]
-            == "Requiring AUTH while not requiring TLS can lead to "
-            "security vulnerabilities!"
-        )
+        for warning in record:
+            if warning.message.args and (
+                warning.message.args[0]
+                == "Requiring AUTH while not requiring TLS can lead to "
+                "security vulnerabilities!"
+            ):
+                break
+            else:
+                pytest.xfail("Did not raise expected warning")
+
         assert caplog.record_tuples[0] == (
             "mail.log",
             logging.WARNING,
@@ -1965,7 +1986,8 @@ class TestAuthArgs:
         ],
     )
     def test_authmechname_decorator_badname(self, name):
-        with pytest.raises(ValueError):
+        expectre = r"Invalid AUTH mechanism name"
+        with pytest.raises(ValueError, match=expectre):
             auth_mechanism(name)
 
 
